/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnenhy@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright © 2002-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

Components.utils.import("resource://mnenhy/mnenhy.js");

// |
// | constants
// |
const gkbInstantApply = document.documentElement.instantApply; // read only once


// |
// | functions
// |

function Startup()
{
  // Since changes do need an application restart anyway, there's no need to do
  // live changes to the manifest in instant apply mode.
  // We just register our update function to the appropriate button.
  window.addEventListener(gkbInstantApply ? "dialogcancel" : "dialogaccept", this.Shutdown, true);
  // read the activation states and update the UI
  Init();
}


function Shutdown()
{
  // Update chrome.manifest.
  Exit();
  // deregister
  window.removeEventListener(gkbInstantApply ? "dialogcancel" : "dialogaccept", this.Shutdown, true);
}


// Read the activation states and update the UI.
function Init()
{
  let oBaseNode = document.getElementById("mnenhyMainPaneCheckboxes");
  let rPackages = goMnenhy.installed.GetAllPackages(true);
  for (let i in rPackages)
  {
    let rPack  = rPackages[i];
    let sPack  = rPack[goMnenhy.installed.kiIndexInfoName];
    let oCheck = oBaseNode.getElementsByAttribute("id", sPack);
    if (oCheck && oCheck.length)
    {
      // The Mnenhy base package is always installed
      // and can never be disabled here (the ID 'mnenhy' must not exist in the panel!).
      // A package state can only be edited if the package is allowed and installed.
      oCheck[0].disabled = !rPack[goMnenhy.installed.kiIndexInfoAllowed]
                        || !rPack[goMnenhy.installed.kiIndexInfoInstall];
      oCheck[0].checked  =  rPack[goMnenhy.installed.kiIndexInfoActive];
    }
  }
}


// Change 'disabled' state according to checkboxes.
function Exit()
{
  // update manifest file to marked changes
  let bSomethingChanged = false;
  let oBaseNode = document.getElementById("mnenhyMainPaneCheckboxes");
  let rPackages = goMnenhy.installed.GetAllPackages(true);
  for (let i in rPackages)
  {
    let rPack  = rPackages[i];
    let sPack  = rPack[goMnenhy.installed.kiIndexInfoName];
    let oCheck = oBaseNode.getElementsByAttribute("id", sPack);
    if (oCheck && oCheck.length)
    {
      // The Mnenhy base package is always installed
      // and can never be disabled here (the ID 'mnenhy' must not exist!).
      // A package state can only be edited if the package is allowed and installed.
      if (    rPack[goMnenhy.installed.kiIndexInfoAllowed]
           && rPack[goMnenhy.installed.kiIndexInfoInstall])
      {
        // we're to allowed to do changes - if any
        let bChecked = oCheck[0].checked;
        let bActive  = rPack[goMnenhy.installed.kiIndexInfoActive];
        if (bChecked != bActive)
        {
          goMnenhy.installed.SetAllowOverlaysForPackage(sPack, bChecked);
          bSomethingChanged = true;
        }
      } // changes allowed
    } // have checkbox
  } // for each known package

  // if something changed, clear the XUL cache as well
  if (bSomethingChanged)
    goMnenhy.installed.DeleteXulFastload();
}


function OnClick(aEvent)
{
  // Look out for a window which defines openUILink -
  // unfortunately, we can't just include eutilityOverlay.js to get it,
  // because the TB version requires a nsIMessenger (which we don't have here)
  // and Firefox' version of that file has a different chrome URL...
  // If no openUILink can be found, try openURL, else give up.
  // So we just play it safe and assume that Mnenhy's prefs are usually not
  // opened standalone (the Extension Manager window provides openURL).
  let oWM = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                      .getService(Components.interfaces.nsIWindowMediator);
  let bDone = false;
  let oWinList = oWM.getEnumerator(null);
  while (oWinList.hasMoreElements())
  {
    let oWin = oWinList.getNext();
    if ("openUILink" in oWin)
    {
      oWin.openUILink(aEvent.target.href, aEvent)
      bDone = true;
      break;
    }
  }
  if (!bDone)
  {
    let oWinList = oWM.getEnumerator(null);
    while (oWinList.hasMoreElements())
    {
      let oWin = oWinList.getNext();
      if ("openURL" in oWin)
      {
        oWin.openURL(aEvent.target.href)
        bDone = true;
        break;
      }
    }
  }
  // if no open* was found, just do nothing
  aEvent.preventDefault(); // must not replace the current window content
}
